package de.uni_mannheim.informatik.dws.ontmatching.kgtrackevaltwo;

import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.Mapping;
import java.util.HashMap;
import java.util.Map;
import java.util.function.ToDoubleFunction;
import java.util.function.ToIntFunction;
import java.util.stream.Stream;


public class AggConfusionMatrix {
    private Map<String, ConfusionMatrix> track;
    private ConfusionMatrix aggregated;
    private ConfusionMatrix aggregatedWithoutEmpty;

    
    public AggConfusionMatrix(){
        this.track = new HashMap<>();
        this.aggregated = null;
        this.aggregatedWithoutEmpty = null;
    }
    
    
    public void addEval(boolean returnedResults, String track, int runtime, Mapping ref, Mapping system){
        addEval(returnedResults, track, runtime, ref, system, true, true);
    }
    public void addEval(boolean returnedResults, String track, int runtime, Mapping ref, Mapping system, boolean kbOneDuplicateFree, boolean kbTwoDuplicateFree){
        this.track.put(track, new ConfusionMatrix(returnedResults, runtime, ref, system, kbOneDuplicateFree, kbTwoDuplicateFree));
    }
    
    
    public void compute(boolean micro){
        if(micro){
            this.aggregated = micro(false);
            this.aggregatedWithoutEmpty =  micro(true);
        }
        else{
            this.aggregated = macro(false);
            this.aggregatedWithoutEmpty =  macro(true);
        }
    }
    
    
    public ConfusionMatrix micro(boolean distinguishEmpty){
        int truePositive = getSummedCmValue(distinguishEmpty, cm->cm.getTruePositive());
        int falsePositive = getSummedCmValue(distinguishEmpty, cm->cm.getFalsePositive());
        int falseNegative = getSummedCmValue(distinguishEmpty, cm->cm.getFalseNegative());

        double precision = Util.divideWithTwoDenominators(truePositive, truePositive, falsePositive);
        double recall = Util.divideWithTwoDenominators(truePositive, truePositive, falseNegative);
        double fmeasure = Util.divideWithTwoDenominators((2.0 * recall * precision), recall, precision);

        double size = getAveragedCmValue(distinguishEmpty, cm->cm.getSystemSize());
        double runtime = getSummedDoubleCmValue(distinguishEmpty, cm->cm.getRuntime());
        
        return new ConfusionMatrix(truePositive,falsePositive, falseNegative, precision, recall, fmeasure, size, runtime, true);
    }
    
    public ConfusionMatrix macro(boolean distinguishEmpty){
        double precision = getAveragedCmValue(distinguishEmpty, cm->cm.getPrecision());
        double recall = getAveragedCmValue(distinguishEmpty, cm->cm.getRecall());
        double fmeasure = Util.divideWithTwoDenominators((2.0 * recall * precision), recall, precision);
        
        double size = getAveragedCmValue(distinguishEmpty, cm->cm.getSystemSize());
        double runtime = getSummedDoubleCmValue(distinguishEmpty, cm->cm.getRuntime());
        
        return new ConfusionMatrix(0, 0, 0, precision, recall, fmeasure, size, runtime, true);
    }
    

    private int getSummedCmValue(boolean distinguishEmpty, ToIntFunction<ConfusionMatrix> mapper){
        return getStream(distinguishEmpty).mapToInt(mapper).sum();
    }
    private double getSummedDoubleCmValue(boolean distinguishEmpty, ToDoubleFunction<ConfusionMatrix> mapper){
        return getStream(distinguishEmpty).mapToDouble(mapper).sum();
    }
    private double getAveragedCmValue(boolean distinguishEmpty, ToDoubleFunction<ConfusionMatrix> mapper){
        return getStream(distinguishEmpty).mapToDouble(mapper).average().orElse(0.0);
    }    
    private Stream<ConfusionMatrix> getStream(boolean distinguishEmpty){
        Stream<ConfusionMatrix> stream = track.values().stream();
        //here we filter all elementa where the whole system alignment is empty
        //(and not only where the alignment of classes, properties or instances are empty)
        if(distinguishEmpty)
            stream = stream.filter(cm->cm.isReturnedResults());//.filter(cm->cm.getSystemSize()==0);
         
        return stream;
    }

    public ConfusionMatrix getCM(String trackName) {
        return track.get(trackName);
    }

    public ConfusionMatrix getAggregated() { return aggregated; }
    public ConfusionMatrix getAggregatedWithoutEmpty() { return aggregatedWithoutEmpty; }
    
    public int numberNotEmpty(){
        return (int) getStream(true).count();
    }
    
    
}
