package de.uni_mannheim.informatik.dws.ontmatching.kgtrackevaltwo;

import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.AlignmentParser;
import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.Mapping;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;
import org.apache.velocity.tools.generic.NumberTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

public class Main {    
    private static final Logger logger = LoggerFactory.getLogger(Main.class);
    private static final String newline = System.getProperty("line.separator");
    
    private static final List<String> testCases = Arrays.asList(
                "darkscape~oldschoolrunescape", "runescape~darkscape", "runescape~oldschoolrunescape",
                "heykidscomics~dc", "marvel~dc", "marvel~heykidscomics",
                "memory-alpha~memory-beta", "memory-alpha~stexpanded", "memory-beta~stexpanded");
    private static final List<String> matchers = Arrays.asList(
            "AML", "POMAP++", "Holontology", "DOME", "LogMap", "LogMapBio", "LogMapLt", "Baseline");//"LogMapLt",
    
    
    public static void main(String[] args) throws IOException{
        logger.info("Start evaluating");
        
        File baseDirReference = new File("C:\\dev\\dbkwik_extraction\\extraction_docker_ubuntu\\newapproach\\e_gold_mapping_interwiki\\gold");
        File baseDirSystem = new File("D:\\oaei\\results_2018");
        
        Map<String, TypeDifferentiation> results = computeResults(baseDirReference, baseDirSystem);
        printAggResults(results);
        
        //Util.repairAlignmentFolder(testCases, new File("D:\\oaei\\results_2018\\LogMap\\"), new File("D:\\oaei\\results_2018\\LogMapRepaired\\"));
        //Util.repairAlignmentFolder(testCases, new File("D:\\oaei\\results_2018\\AML\\"), new File("D:\\oaei\\results_2018\\AMLRepaired\\"));
        //Util.repairAlignmentFolder(testCases, new File("D:\\oaei\\results_2018\\LogMapLt_old\\"), new File("D:\\oaei\\results_2018\\LogMapLt\\"));
    }
    
    
    
    private static Map<String, TypeDifferentiation> computeResults(File baseDirReference, File baseDirSystem){
        Map<String, TypeDifferentiation> resultsPerMatcher = new HashMap<>();
        for(String matcher : matchers){
            File matcherResultsDir = new File(baseDirSystem, matcher);
            TypeDifferentiation eval = new TypeDifferentiation();
            for(String testCase : testCases){
                Mapping refAlignment;                
                try {
                    refAlignment = AlignmentParser.parse(new FileInputStream(new File(baseDirReference, testCase + "~evaluation.xml")));
                } catch (IOException|SAXException ex) {
                    logger.error("No reference");
                    continue;
                }
                Mapping systemAlignment = new Mapping();
                int runtime = 0;
                try{
                    systemAlignment = AlignmentParser.parse(new FileInputStream(new File(matcherResultsDir, testCase + "~results.xml")));
                    runtime = Util.getRuntimeSeconds(new File(matcherResultsDir, testCase + "~time.txt"));
                }catch (IOException|SAXException ex) {
                    logger.info("No system or time file available for " + matcher + " and track " + testCase);
                }
                
                systemAlignment = Util.checkDirectionOfMapping(testCase, systemAlignment);
                eval.addEval(testCase, runtime, refAlignment, systemAlignment);
            }
            eval.compute(true);
            resultsPerMatcher.put(matcher, eval);
        }
        return resultsPerMatcher;
    }
    
    private static void printAggResults(Map<String, TypeDifferentiation> results) throws IOException{
        Velocity.setProperty("resource.loader", "classpath");
        Velocity.setProperty("classpath.resource.loader.class", ClasspathResourceLoader.class.getName());        
        Velocity.init();
        VelocityContext context = new VelocityContext();
        context.put("results", results);
        context.put("testCases", testCases);
        context.put("matchers", matchers);
        context.put("numberTool", new NumberTool());
        context.put("locale", Locale.US);
        context.put("Double", Double.class);
        Template template = Velocity.getTemplate("aggresults.vm");
        
        Writer writer = new FileWriter(new File("index.html"));
        template.merge( context, writer );
        writer.flush();
        writer.close();
        
        //StringWriter sw = new StringWriter();
        //template.merge( context, sw );
        //sw.toString()
    }
    
}
