package de.uni_mannheim.informatik.dws.ontmatching.kgtrackevaltwo;

import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.Mapping;
import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.MappingCell;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class TypeDifferentiation {
    
    private AggConfusionMatrix classes;
    private AggConfusionMatrix properties;
    private AggConfusionMatrix instances;
    private AggConfusionMatrix overall;
    
    public TypeDifferentiation(){
        this.classes = new AggConfusionMatrix();
        this.properties = new AggConfusionMatrix();
        this.instances = new AggConfusionMatrix();
        this.overall = new AggConfusionMatrix();
    }
    
    public void addEval(String track, int runtime, Mapping ref, Mapping system){
        addEval(track, runtime, ref, system, true, true);
    }
    public void addEval(String track, int runtime, Mapping ref, Mapping system, boolean kbOneDuplicateFree, boolean kbTwoDuplicateFree){

        Mapping refClass = new Mapping();
        Mapping refProp = new Mapping();
        Mapping refInst = new Mapping();
        Mapping refUnclear = new Mapping();
                
        getAlignmentTypes(ref, refClass, refProp, refInst, refUnclear);
        
        Mapping systemClass = new Mapping();
        Mapping systemProp = new Mapping();
        Mapping systemInst = new Mapping();
        Mapping systemUnclear = new Mapping();
        
        getAlignmentTypes(system, systemClass, systemProp, systemInst, systemUnclear);
        
        boolean systemReturnedResults = system.size() != 0;
        
        
        classes.addEval(systemReturnedResults, track, runtime, refClass, systemClass, kbOneDuplicateFree, kbTwoDuplicateFree);
        properties.addEval(systemReturnedResults, track, runtime, refProp, systemProp, kbOneDuplicateFree, kbTwoDuplicateFree);
        instances.addEval(systemReturnedResults, track, runtime, refInst, systemInst, kbOneDuplicateFree, kbTwoDuplicateFree);
        overall.addEval(systemReturnedResults, track, runtime, ref, system, kbOneDuplicateFree, kbTwoDuplicateFree);

    }
    
    public void compute(boolean micro){
        this.classes.compute(micro);
        this.properties.compute(micro);
        this.instances.compute(micro);
        this.overall.compute(micro);
    }
    
    private static final Set<String> classTypes = new HashSet(Arrays.asList("class", "ontology", "null"));
    private static final Set<String> propTypes = new HashSet(Arrays.asList("property", "null"));
    private static final Set<String> instanceTypes = new HashSet(Arrays.asList("resource", "null"));
    private static final Pattern p = Pattern.compile("http:\\/\\/dbkwik\\.webdatacommons\\.org\\/.+?\\/(.+?)\\/.*");
    private static void getAlignmentTypes(Mapping m, Mapping clazz, Mapping props, Mapping instance, Mapping unclear){
        for(MappingCell c : m){
            String sourceType = getType(c.getEntityOne());
            String targetType = getType(c.getEntityTwo());
            
            if(classTypes.contains(sourceType) && classTypes.contains(targetType)){
                clazz.add(c);
            }else if (propTypes.contains(sourceType) && propTypes.contains(targetType)){
                props.add(c);
            }else if (instanceTypes.contains(sourceType) && instanceTypes.contains(targetType)){
                instance.add(c);
            }
            else{
                unclear.add(c);
            }
        }
    }
    private static String getType(String uri){
        if(uri.equals("null"))
            return "null";
        Matcher matcher = p.matcher(uri);
        if (matcher.find())
            return matcher.group(1);
        return "";
    }

    public AggConfusionMatrix getClasses() {return classes;}
    public AggConfusionMatrix getProperties() { return properties;}
    public AggConfusionMatrix getInstances() {return instances;}
    public AggConfusionMatrix getOverall() {return overall;}
    
    public AggConfusionMatrix getCMAggByType(String type){
        switch (type) {
            case "class":
                return classes;
            case "property":
                return properties;
            case "instance":
                return instances;
            case "overall":
                return overall;
            default:
                throw new IllegalArgumentException("Invalid type");
        }
    }
}
