package de.uni_mannheim.informatik.dws.ontmatching.kgtrackevaltwo;

import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.Mapping;
import de.uni_mannheim.informatik.dws.ontmatching.yetanotheralignmentapi.MappingCell;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CodingErrorAction;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Util {
    private static final Logger logger = LoggerFactory.getLogger(Util.class);
    private static final String newline = System.getProperty("line.separator");
    
        public static Mapping checkDirectionOfMapping(String testCase, Mapping m){        
        String[] caseSplit =  testCase.split("~");
        String left = caseSplit[0];
        String right = caseSplit[1];
        if(isCorrectDirection(left, right, m)){
            return m;
        }
        else{
            Mapping swappedMapping = new Mapping();
            for(MappingCell c : m){
                swappedMapping.add(c.getEntityTwo(), c.getEntityOne(), c.getConfidence(), c.getRelation());
            }
            return swappedMapping;
        }
    }
    private static boolean isCorrectDirection(String left, String right, Mapping m){
        for(MappingCell c : m){
            if(c.getEntityOne().contains(left) && c.getEntityTwo().contains(right)){
                return true;
            }
            if(c.getEntityOne().contains(right) && c.getEntityTwo().contains(left)){
                return false;
            }
        }
        return true;
    }
    
    public static double divideWithTwoDenominators(double numerator, double denominatorOne, double denominatorTwo){
        if ((denominatorOne + denominatorTwo) > 0.0) {
            return numerator / (denominatorOne + denominatorTwo);
        }
        else{
            return 0.0;
        }
    }
    
    private static final Pattern timepattern = Pattern.compile("real\\s*(\\d*)m(\\d*).(\\d*)s");
    public static int getRuntimeSeconds(File file) throws FileNotFoundException, IOException{
        try (BufferedReader br = new BufferedReader(new FileReader(file))) {
            String line;
            while ((line = br.readLine()) != null) {
                Matcher matcher = timepattern.matcher(line);
                if (matcher.find()){
                    int minutes = Integer.parseInt(matcher.group(1));
                    int seconds = Integer.parseInt(matcher.group(2));
                    return (minutes * 60 ) + seconds;
                }                    
            }
        }
        logger.error("did not find the time in file {}", file);
        return 0;
    }
    
    
    public static void repairAlignmentFolder(List<String> testCases, File matcherResultsDirIn, File matcherResultsDirOut){
        Charset charset = Charset.forName("UTF-8");
        CharsetDecoder utf8Decoder = Charset.forName("UTF-8").newDecoder();
        utf8Decoder.onMalformedInput(CodingErrorAction.IGNORE);
        utf8Decoder.onUnmappableCharacter(CodingErrorAction.IGNORE);
        for(String testCase : testCases){            
            try (BufferedReader brIn = new BufferedReader(new InputStreamReader(new FileInputStream(new File(matcherResultsDirIn, testCase + "~results.xml")), StandardCharsets.UTF_8));
                OutputStreamWriter writer = new OutputStreamWriter(new FileOutputStream(new File(matcherResultsDirOut, testCase + "~results.xml")), StandardCharsets.UTF_8)) {
                String line;
                while ((line = brIn.readLine()) != null) {
                    line = line.replace("&", "&amp;");
                    writer.write(line);
                    writer.write(newline);
                }
            } catch (IOException ex) {
                logger.info("error repairing ", ex);
            }
        }
    }
    
}
